#ifndef __REALMLOCATION_H
	#define __REALMLOCATION_H

#include "XYLocation.h"
#include "HexSpiralCoordinate.h"

class RealmLocation {

	// object to define and and contain location data within the realms

	private:

	// HexSpiralCoordinate objects stores computation-intensive values about hex grid location based on the HexIndex
	// only needs to be recalculated whenever each hex index num changes
	// and when needing to calculate another coordinate relative to an adjacent hex
	// whose location info attributes have already been calculated,
	// can even easily be derived more efficiently requiring only integer math
	HexSpiralCoordinate

	// 7 Realms in the world...  Realm 0 at the center (power 11)
	RealmHexInfo,

	// Within each Realm, 7 Regions (power 10)
	RegionHexInfo,

	// Within each Region, 547 Zones, similarly contained in a 13-hex Zone radius.  Zone 0 is the center of the region.
	// (power 8)
	ZoneHexInfo,

	// Within each Zone, 547 Scenes
	// (power 4)
	SceneHexInfo,

	// Within each Scene, another 13 hex radius space grid, containing 547 tactical spaces, each imagined as being roughly 1 meter in diameter
	// (power 1)
	SpaceHexInfo;

	// more compacted / bare-bones storage of above data:
	// but would it really be more efficient?
	// to rebuild the HexSpiralCoordinate data at each use / instantiation?
	// hmmmmmm......
//	RealmSceneKey SceneLocation;
//	long int SpaceIndex;

	// internal / private only: returns reference to relevant HexSpiralCoordinate
	HexSpiralCoordinate& getLevelCoordinate(int level);

	public:

	RealmLocation(long int realm=0, long int region=0, long int zone=0, long int scene=0, long int space=0);

	// for external use, returns HexSpiralCoordinate for given level by value
	HexSpiralCoordinate getHexSpiralCoordinate(int level) const;

	void setHexIndex(long int n, int level);
	long int getHexIndex(int level);

	void setRealm(long int n) { RealmHexInfo.setHexIndex(n); }
	void setRegion(long int n) { RegionHexInfo.setHexIndex(n); }
	void setZone(long int n) { ZoneHexInfo.setHexIndex(n); }
	void setScene(long int n) { SceneHexInfo.setHexIndex(n); }
	void setSpace(long int n) { SpaceHexInfo.setHexIndex(n); }

	void setSpaceCoordinate(HexSpiralCoordinate c) { SpaceHexInfo = c; }

	long int getRealm() { return RealmHexInfo.getHexIndex(); }
	long int getRegion() { return RegionHexInfo.getHexIndex(); }
	long int getZone() { return ZoneHexInfo.getHexIndex(); }
	long int getScene() { return SceneHexInfo.getHexIndex(); }
	long int getSpace() { return SpaceHexInfo.getHexIndex(); }


	void moveSpace(int direction);
	void moveScene(int direction);
	void moveZone(int direction);
	void moveRegion(int direction);
	void moveRealm(int direction);

	static bool isInvertedGrid(int view_mode);
	double getZoomScale(double level);

	double getXScaleFactor(int view_mode);
	double getYScaleFactor(int view_mode);

	XYLocation getRealmWorldCoordinate(int zoom_precision = 1);

	// returns xy location of current index at a given zoom level
	// (in hex-movement units, relative to center of current zoom-grid)
	XYLocation getXY(int zoomlevel);



	// if a space is outside the boundaries of the zone it is relative to
	// relocate it to a roughly appropriate space within the zone it is within
	// use a similar procedure for zones relative to regions
	void normalize();
};

#endif // __REALMLOCATION_H
